/**
 * Copyright (c) 2008-2021, OPPO Mobile Comm Corp., Ltd
 * VENDOR_EDIT
 * @file heythings_task.c
 * @brief heythings application entry
 * @version 0.1
 * @date 2021-03-12
 * @author WatWu
 */

#include <string.h>

#include "heythings.h"
#include "heythings_task.h"
#include "schema/softwareUpdate/software_update_service.h"
#include "schema/light/light_service.h"
#include "schema/devDiagnosis/dev_diagnosis_service.h"
#include "user_task.h"
#include "cli_task.h"
#include "app_version.h"

static void device_services_init(void)
{
    int ret;

    struct light_properties light_props = { 0 };
    struct light_rgb rgb = { 0 };
    light_props.colorrgb = &rgb;
    ret = light_service_init(&light_props);
    if (ret != 0) {
        // do somethings
    }

    struct dev_diagnosis_properties dev_diagnosis_props = { 0 };
    ret = dev_diagnosis_service_init(&dev_diagnosis_props);
    if (ret != 0) {
        // do somethings
    }

    struct software_update_properties soft_update_props = { 0 };
    soft_update_props.currentsoftwareversion = FIRMWARE_VERSION;
    soft_update_props.updatestate = SOFTWARE_UPDATE_UPDATE_STATE_IDLE;
    soft_update_props.updateprogress = 0;
    // soft_update_props...
    // assign initial values ​​to properties.
    // ...
    ret = software_update_service_init(&soft_update_props);
    if (ret != 0) {
        // do somethings
    }
}

static void sdk_state_callback(enum hey_sdk_state state)
{
    switch (state) {
    case HEY_SDK_STATE_WIFI_CONNECTING:
        hey_log_printf(HEY_LOG_LEVEL_INFO, "sdk callback state WIFI_CONNECTING");
        break;
    case HEY_SDK_STATE_SERVER_CONNECTING:
        hey_log_printf(HEY_LOG_LEVEL_INFO, "sdk callback state SERVER_CONNECTING");
        break;
    case HEY_SDK_STATE_SERVER_CONNECTED:
        hey_log_printf(HEY_LOG_LEVEL_INFO, "sdk callback state SERVER_CONNECTED");
        break;
    default:
        break;
    }
}

static void sdk_time_synced(void)
{
    //Time is synchronized
    hey_log_printf(HEY_LOG_LEVEL_INFO, "sdk callback Time is synchronized");
}

static void setup_reset_cb(void)
{
    hey_log_printf(HEY_LOG_LEVEL_INFO, "sdk callback setup reset by the app");
    esp_restart();
}

const struct hey_sdk_callbacks sdk_callbacks = {
    .state_callback = sdk_state_callback,
    .time_synced = sdk_time_synced,
    .setup_reset = setup_reset_cb
};

static int heythings_run(void)
{
    struct hey_sdk_config conf = {
        .info = {
                .brand = "blicca",
                .manufacture = "oppo",
                .hard_version = "1.0.0",
                .soft_version = FIRMWARE_VERSION,
                .model = "f001",
        }
    };
    conf.cbs = &sdk_callbacks;

    int ret = hey_sdk_init(&conf);
    if (ret != 0) {
        // do somethings
    }
    device_services_init();

    // user task， start after hey_sdk_init()
    xTaskCreate(&user_task, "user", 6 * 1024, NULL, 4, NULL);
    xTaskCreate(&cli_task, "cli", 4 * 1024, NULL, 3, NULL);

    printf("application version : %s\n", conf.info.soft_version);

    // run heythings sdk and will never return if everything OK.
    return hey_sdk_run();
}

void heythings_task(void *pvParameter)
{
    heythings_run();
    vTaskDelete(NULL);
}
