/**
 * Copyright (c) 2008-2021, OPPO Mobile Comm Corp., Ltd
 * VENDOR_EDIT
 * @file dev_diagnosis_service.c
 * @brief
 * @version
 * @date 2021-4-6
 * @author
 */

#include <errno.h>

#include "devDiagnosis.pb-c.h"
#include "dev_diagnosis_service.h"
#include "hey_schema_api.h"

#define DEV_DIAGNOSIS_SERVICE 64000
#define PROP_IID_CNT 1
#define ACT_IID_CNT 1

static const uint32_t dev_diagnosis_properties_iids[] = { 1 };
static struct hey_service *dev_diagnosis_service;

static int action_upload_log_cmd_call_cb(void *cb_data, struct hey_action_ctx *ctx, size_t len, unsigned char *value, void **user_context)
{
    int ret = -1;
    Iot__Service__DevDiagnosis__ActionInUploadLogCmd *upload_log_cmd = iot__service__dev_diagnosis__action_in_upload_log_cmd__unpack(NULL, len, value);
    if (NULL == upload_log_cmd) {
        hey_action_stop(ctx);
        return EINVAL;
    }

    struct dev_diagnosis_upload_log_cmd_in action;
    ret = dev_diagnosis_upload_log_cmd_call_cb(ctx, &action);

    iot__service__dev_diagnosis__action_in_upload_log_cmd__free_unpacked(upload_log_cmd, NULL);
    return ret;
}

static int service_dev_diagnosis_properties_write_cb(void *cb_data, size_t n_iid, unsigned int *iids,
                                                     size_t len, unsigned char *value)
{
    Iot__Service__DevDiagnosis__Properties *props = iot__service__dev_diagnosis__properties__unpack(NULL, len, value);
    if (NULL == props) {
        return EINVAL;
    }

    struct dev_diagnosis_properties properties;
    properties.enable = props->enable;
    int ret = dev_diagnosis_properties_write_cb(n_iid, iids, &properties);

    iot__service__dev_diagnosis__properties__free_unpacked(props, NULL);
    return ret;
}

static const struct hey_properties_callbacks props_cbs = {
    .write = service_dev_diagnosis_properties_write_cb
};

static const struct hey_action_ctx_callbacks upload_log_cmd_cbs = {
    .start = action_upload_log_cmd_call_cb
};

static int dev_diagnosis_service_register(Iot__Service__DevDiagnosis__Properties *props)
{
    unsigned char *buf;
    size_t buf_len;
    buf_len = iot__service__dev_diagnosis__properties__get_packed_size(props);
    if (buf_len == 0) {
        buf = NULL;
    } else {
        buf = malloc(buf_len);
        if (!buf) {
            return ENOMEM;
        }
        iot__service__dev_diagnosis__properties__pack(props, buf);
    }

    struct hey_action_entry *actions_iids[ACT_IID_CNT];
    uint8_t actions_n_iid = 0;

    struct hey_action_entry act_upload_log_cmd;
    act_upload_log_cmd.iid = DEV_DIAGNOSIS_ACTION_UPLOAD_LOG_CMD;
    act_upload_log_cmd.type = HEY_ACTION_ONCE;
    act_upload_log_cmd.cbs = &upload_log_cmd_cbs;
    actions_iids[actions_n_iid++] = &act_upload_log_cmd;

    struct hey_schema_service schema_srv;
    hey_schema_service_init(&schema_srv);
    schema_srv.properties.iids = (uint32_t *)dev_diagnosis_properties_iids;
    schema_srv.properties.n_iid = PROP_IID_CNT;
    schema_srv.properties.value = buf;
    schema_srv.properties.len = buf_len;
    schema_srv.properties.cbs = &props_cbs;
    schema_srv.actions.iids = *actions_iids;
    schema_srv.actions.n_iid = ACT_IID_CNT;
    dev_diagnosis_service = hey_service_register(DEV_DIAGNOSIS_SERVICE, &schema_srv, NULL);
    if (NULL == dev_diagnosis_service) {
        free(buf);
        return errno;
    }

    free(buf);
    return 0;
}

/**
 * @brief update properties of dev diagnosis service
 * @param count      [the number of properties to be updated]
 * @param enumerate  [the iid enumeration of properties to be updated]
 * @param properties [properties structure with updated content]
 * @return int       [update result]
 */
int dev_diagnosis_properties_update(size_t count, unsigned int *enumerate, struct dev_diagnosis_properties *properties)
{
    if (NULL == properties) {
        return EINVAL;
    }

    int ret = 0;
    Iot__Service__DevDiagnosis__Properties *props = NULL;
    unsigned char *buf = NULL;
    props = malloc(sizeof(Iot__Service__DevDiagnosis__Properties));
    if (NULL == props) {
        ret = ENOMEM;
        goto out;
    }
    iot__service__dev_diagnosis__properties__init(props);
    props->enable = properties->enable;

    size_t buf_len;
    buf_len = iot__service__dev_diagnosis__properties__get_packed_size(props);
    if (buf_len > 0) {
        buf = malloc(buf_len);
        if (NULL == buf) {
            ret = ENOMEM;
            goto out;
        }
        iot__service__dev_diagnosis__properties__pack(props, buf);
    }

    struct hey_properties_update_content content = {
        .n_iid = count,
        .iids = enumerate,
        .len = buf_len,
        .value = buf
    };
    ret = hey_properties_update(dev_diagnosis_service, &content);

out:
    free(props);
    free(buf);
    return ret;
}

/**
 * @brief dev diagnosis service initialization
 * @param properties [dev diagnosis service properties]
 * @return int       [initialize result]
*/
int dev_diagnosis_service_init(const struct dev_diagnosis_properties *properties)
{
    int ret = 0;
    Iot__Service__DevDiagnosis__Properties *props = NULL;
    props = malloc(sizeof(Iot__Service__DevDiagnosis__Properties));
    if (NULL == props) {
        ret = ENOMEM;
        goto out;
    }
    iot__service__dev_diagnosis__properties__init(props);
    if (NULL != properties) {
        props->enable = properties->enable;
    }

    ret = dev_diagnosis_service_register(props);

out:
    free(props);
    return ret;
}

/**
 * @brief upload log cmd action return
 * @param ctx    [action context]
 * @param action [upload log cmd action out]
 * @return int   [action return result, 0 means success]
 */
int dev_diagnosis_upload_log_cmd_return(struct hey_action_ctx *ctx, struct dev_diagnosis_upload_log_cmd_out *action)
{
    int ret = 0;
    ret = hey_action_msg_send(ctx, 0, NULL);
    hey_action_stop(ctx);
    return ret;
}
