#ifndef DEV_DIAGNOSIS_SERVICE_H
#define DEV_DIAGNOSIS_SERVICE_H

#include "hey_schema_common.h"

struct dev_diagnosis_upload_log_cmd_in {
    char __place_holder__; // make compiler happy
};

struct dev_diagnosis_upload_log_cmd_out {
    char __place_holder__; // make compiler happy
};

enum DEV_DIAGNOSIS_PROPERTY {
    DEV_DIAGNOSIS_PROPERTY_ENABLE = 1
};

enum DEV_DIAGNOSIS_ACTION {
    DEV_DIAGNOSIS_ACTION_UPLOAD_LOG_CMD = 2
};

struct dev_diagnosis_properties {
    bool enable;
};

/**
 * @brief dev diagnosis service initialization
 * @param properties [dev diagnosis service properties]
 * @return int       [initialize result]
*/
int dev_diagnosis_service_init(const struct dev_diagnosis_properties *properties);

/**
 * @brief properties write callback of dev diagnosis service
 * @param count      [the number of properties will be written]
 * @param enumerate  [the iid enumeration of properties will be written]
 * @param properties [properties structure with written content]
 * @return int       [be writed result, 0 means success]
*/
int dev_diagnosis_properties_write_cb(size_t count, unsigned int *enumerate, struct dev_diagnosis_properties *properties);

/**
 * @brief update properties of dev diagnosis service
 * @param count      [the number of properties to be updated]
 * @param enumerate  [the iid enumeration of properties to be updated]
 * @param properties [properties structure with updated content]
 * @return int       [update result]
 */
int dev_diagnosis_properties_update(size_t count, unsigned int *enumerate, struct dev_diagnosis_properties *properties);

/**
 * @brief upload log cmd action call callback of dev diagnosis service
 * @param ctx    [action context]
 * @param action [upload log cmd action in]
 * @return int   [action call handle result, 0 means success]
 */
int dev_diagnosis_upload_log_cmd_call_cb(struct hey_action_ctx *ctx, struct dev_diagnosis_upload_log_cmd_in *action);

/**
 * @brief upload log cmd action return
 * @param ctx    [action context]
 * @param action [upload log cmd action out]
 * @return int   [action return result, 0 means success]
 */
int dev_diagnosis_upload_log_cmd_return(struct hey_action_ctx *ctx, struct dev_diagnosis_upload_log_cmd_out *action);

#endif /* DEV_DIAGNOSIS_SERVICE_H */
