#ifndef LIGHT_SERVICE_H
#define LIGHT_SERVICE_H

#include "hey_schema_common.h"

enum light_action_light_language_mode {
    LIGHT_ACTION_LIGHT_LANGUAGE_MODE_MISS_YOU = 0,
    LIGHT_ACTION_LIGHT_LANGUAGE_MODE_SLEEP_NOW = 1,
    LIGHT_ACTION_LIGHT_LANGUAGE_MODE_ALMOST_HOME = 2
};

enum light_light_mode {
    LIGHT_LIGHT_MODE_MANUAL = 0,
    LIGHT_LIGHT_MODE_READING = 1,
    LIGHT_LIGHT_MODE_SCREEN_READING = 2,
    LIGHT_LIGHT_MODE_SLEEP = 3,
    LIGHT_LIGHT_MODE_LEISURE = 4
};

struct light_rgb {
    uint32_t r;
    uint32_t g;
    uint32_t b;
};

struct light_light_language_in {
    enum light_action_light_language_mode mode;
};

struct light_light_language_out {
    char __place_holder__; // make compiler happy
};

struct light_free_control_night_light {
    bool enable;
    uint32_t starthour;
    uint32_t startmin;
    uint32_t endhour;
    uint32_t endmin;
};

struct light_scene_ctt_brightness {
    enum light_light_mode lightmode;
    uint32_t colortemp;
    uint32_t brightness;
};

struct light_study_mode_setting {
    uint32_t repeatnum;
    uint32_t studytime;
    uint32_t breaktime;
};

enum LIGHT_PROPERTY {
    LIGHT_PROPERTY_POWER = 1,
    LIGHT_PROPERTY_BRIGHTNESS = 2,
    LIGHT_PROPERTY_CCT = 3,
    LIGHT_PROPERTY_LIGHTMODE = 4,
    LIGHT_PROPERTY_AUTOCOLORTEMPERATURE = 5,
    LIGHT_PROPERTY_COLORRGB = 6,
    LIGHT_PROPERTY_COLORSATURATION = 7,
    LIGHT_PROPERTY_AUTOBRIGHTNESS = 9,
    LIGHT_PROPERTY_FREECONTROLNIGHTLIGHT = 11,
    LIGHT_PROPERTY_SCENECTTBRIGHTNESS = 12,
    LIGHT_PROPERTY_COLORCTTMODE = 13,
    LIGHT_PROPERTY_STUDYMODESETTING = 14,
    LIGHT_PROPERTY_APPOINTMENTPOWERON = 15,
    LIGHT_PROPERTY_APPOINTMENTPOWEROFF = 16
};

enum LIGHT_ACTION {
    LIGHT_ACTION_LIGHT_LANGUAGE = 8
};

struct light_properties {
    bool power;
    uint32_t brightness;
    uint32_t cct;
    enum light_light_mode lightmode;
    bool autocolortemperature;
    struct light_rgb *colorrgb;
    uint32_t colorsaturation;
    bool autobrightness;
    struct light_free_control_night_light *freecontrolnightlight;
    size_t n_scenecttbrightness;
    struct hey_array_node **scenecttbrightness;
    bool colorcttmode;
    struct light_study_mode_setting *studymodesetting;
    uint32_t appointmentpoweron;
    uint32_t appointmentpoweroff;
};

/**
 * @brief light service initialization
 * @param properties [light service properties]
 * @return int       [initialize result]
*/
int light_service_init(const struct light_properties *properties);

/**
 * @brief properties write callback of light service
 * @param count      [the number of properties will be written]
 * @param enumerate  [the iid enumeration of properties will be written]
 * @param properties [properties structure with written content]
 * @return int       [be writed result, 0 means success]
*/
int light_properties_write_cb(size_t count, unsigned int *enumerate, struct light_properties *properties);

/**
 * @brief update properties of light service
 * @param count      [the number of properties to be updated]
 * @param enumerate  [the iid enumeration of properties to be updated]
 * @param properties [properties structure with updated content]
 * @return int       [update result]
 */
int light_properties_update(size_t count, unsigned int *enumerate, struct light_properties *properties);

/**
 * @brief light language action call callback of light service
 * @param ctx    [action context]
 * @param action [light language action in]
 * @return int   [action call handle result, 0 means success]
 */
int light_light_language_call_cb(struct hey_action_ctx *ctx, struct light_light_language_in *action);

/**
 * @brief light language action return
 * @param ctx    [action context]
 * @param action [light language action out]
 * @return int   [action return result, 0 means success]
 */
int light_light_language_return(struct hey_action_ctx *ctx, struct light_light_language_out *action);

#endif /* LIGHT_SERVICE_H */
