#include "heythings.h"
#include "schema/light/light_service.h"

/**
 * @brief properties write callback of light service
 * @param count      [the number of properties will be written]
 * @param enumerate  [the iid enumeration of properties will be written]
 * @param properties [properties structure with written content]
 * @return int       [be writed result, 0 means success]
 */
int light_properties_write_cb(size_t count, unsigned int *enumerate, struct light_properties *properties)
{
    // This is handled by the application layer
    for (size_t i = 0; i < count; i++) {
        switch (enumerate[i]) {
        case LIGHT_PROPERTY_POWER:
            hey_log_printf(HEY_LOG_LEVEL_INFO, "set switch : %d", properties->power);
            break;
        case LIGHT_PROPERTY_BRIGHTNESS:
            hey_log_printf(HEY_LOG_LEVEL_INFO, "set brightness : %d", properties->brightness);
            break;
        case LIGHT_PROPERTY_CCT:
            hey_log_printf(HEY_LOG_LEVEL_INFO, "set colortemp : %d", properties->cct);
            break;
        case LIGHT_PROPERTY_LIGHTMODE:
            hey_log_printf(HEY_LOG_LEVEL_INFO, "set mode : %d", properties->lightmode);
            break;
        default:
            break;
        }
    }
    light_properties_update(count, enumerate, properties);

    return 0;
}

/**
 * @brief light language action call callback of light service
 * @param ctx    [action context]
 * @param action [light language action in]
 * @return int   [action call handle result, 0 means success]
 */
int light_light_language_call_cb(struct hey_action_ctx *ctx, struct light_light_language_in *action)
{
    // This is handled by the application layer
    // ctx needs to be saved by yourself if you call return asynchronously

    return -1;
}
