#include "schema/softwareUpdate/software_update_service.h"

extern int start_ota_with_url(char *url, bool sign_flag, unsigned char *sign, int sign_len);

/**
 * @brief start update action call callback of software update service
 * @param ctx    [action context]
 * @param action [start update action in]
 * @return int   [action call handle result, 0 means success]
 */
int software_update_start_update_call_cb(struct hey_action_ctx *ctx, struct software_update_start_update_in *action)
{
    // This is handled by the application layer
    // ctx needs to be saved by yourself if you call return asynchronously
    int ret = start_ota_with_url(action->packageurl, true, action->signatures.data, action->signatures.len);
    struct software_update_start_update_out action_out;
    software_update_start_update_return(ctx, &action_out);

    return ret;
}

/**
 * @brief start update file action start callback of software update service
 * @param ctx          [action context]
 * @param action       [start update file action in]
 * @param user_context [user context record in SDK as needed, otherwise set it NULL]
 * @return int         [action start handle result, 0 means success]
 */
int software_update_start_update_file_start_cb(struct hey_action_ctx *ctx, struct software_update_start_update_file_in *action, void **user_context)
{
    // This is handled by the application layer
    // ctx needs to be saved by yourself
    // user_context can be assigned to SDK to record it for you as needed

    return -1;
}

/**
 * @brief start update file action message receive callback of software update service
 * @param user_context [callback data which form action start callback function]
 * @param action       [start update file action in]
 * @return int         [action message callback handle result, 0 means success]
 */
int software_update_start_update_file_message_cb(void *user_context, struct software_update_start_update_file_in *action)
{
    // This is handled by the application layer
    // user_context is assigned to the SDK in the *_start_cb function

    return -1;
}

/**
 * @brief start update file action stop callback of software update service
 *        closed by the peer, user must to call action stop function
 * @param user_context [callback data which form action start callback function]
 * @return none
 */
void software_update_start_update_file_stop_cb(void *user_context)
{
    // This is handled by the application layer
    /* User must call action stop function here !!!
     * for example: software_update_start_update_file_stop(ctx);
     */
}

/**
 * @brief query new update action call callback of software update service
 * @param ctx    [action context]
 * @param action [query new update action in]
 * @return int   [action call handle result, 0 means success]
 */
int software_update_query_new_update_call_cb(struct hey_action_ctx *ctx, struct software_update_query_new_update_in *action)
{
    // This is handled by the application layer
    // ctx needs to be saved by yourself if you call return asynchronously

    return -1;
}
