#include <string.h>

#include "user_task.h"
#include "heythings.h"
#include "nvs_flash.h"

#define KEY_BIN_FILE_HEADER 140
#define BASEINFO_PARTITION_NAME "dev_base"
static const esp_partition_t *p_partition;

static int esp32_storage_read_seek(unsigned int offset, unsigned char *buf, unsigned int size)
{
    int ret = esp_partition_read(p_partition, offset, buf, size);
    if (ret != ESP_OK) {
        hey_log_printf(HEY_LOG_LEVEL_ERR, "error:%d, offset:%d", ret, offset);
    }

    return ret;
}

static void load_key(unsigned int *offset, char **buf, size_t *len, const char *start_str, const char *end_str)
{
    char *tmp_buf = NULL;
    char *start = NULL;
    char *end = NULL;
    tmp_buf = malloc(1024);
    if (NULL == tmp_buf) {
        hey_log_printf(HEY_LOG_LEVEL_ERR, "Failed to allocate mem");
        return;
    }

    int ret = esp32_storage_read_seek(*offset, (unsigned char *)tmp_buf, 1024);

    if (strstr(tmp_buf, start_str) && strstr(tmp_buf, end_str)) {
        start = strstr(tmp_buf, start_str);
        end = strstr(tmp_buf, end_str) + strlen(end_str) + 2; // '\n' + '\0'
        char *p = malloc(end - start);
        memcpy(p, start, end - start - 1);
        p[end - start - 1] = '\0';
        *buf = p;
    }
    if (*buf) {
        *offset += (end - start);
        *len = end - start;
    } else {
        hey_log_printf(HEY_LOG_LEVEL_DEBUG, "#ret = %d, buf_len = %d \r\n%s\r\n", ret, strlen(tmp_buf), tmp_buf);
    }

    free(tmp_buf);
}

static void setup_provision_confirm_cb(void)
{
    //later could call hey_setup_confirm(HEY_SETUP_ACCEPT);
    hey_log_printf(HEY_LOG_LEVEL_WARN, "Please confirm setup !");
}

static void setup_state_cb(enum hey_setup_state state)
{
    hey_log_printf(HEY_LOG_LEVEL_INFO, "Setup state : %d", state);
}

static void setup_set_pin_cb(char **pin, size_t *pin_len)
{
}

static const struct hey_setup_callbacks setup_cbs = {
    .provision_confirm = setup_provision_confirm_cb,
    .set_pin = setup_set_pin_cb,
    .state_callback = setup_state_cb
};

void device_start_to_setup(void)
{
    static char *dev_private_key = NULL;
    static size_t dev_private_key_len = 0;
    static char *dev_cert = NULL;
    static size_t dev_cert_len = 0;
    static char *ecdh_private_key = NULL;
    static size_t ecdh_private_key_len = 0;
    static char *product_cert = NULL;
    static size_t product_cert_len = 0;

    unsigned int offset = KEY_BIN_FILE_HEADER;

    //heythings SDK init flash already
    p_partition = esp_partition_find_first(64, ESP_PARTITION_SUBTYPE_ANY, BASEINFO_PARTITION_NAME);
    if (!p_partition) {
        hey_log_printf(HEY_LOG_LEVEL_ERR, "Partition '%s' doesn't exists", BASEINFO_PARTITION_NAME);
    }
    load_key(&offset, &dev_private_key, &dev_private_key_len, "-----BEGIN EC PRIVATE KEY-----", "-----END EC PRIVATE KEY-----");
    load_key(&offset, &dev_cert, &dev_cert_len, "-----BEGIN CERTIFICATE-----", "-----END CERTIFICATE-----");
    load_key(&offset, &ecdh_private_key, &ecdh_private_key_len, "-----BEGIN EC PRIVATE KEY-----", "-----END EC PRIVATE KEY-----");
    load_key(&offset, &product_cert, &product_cert_len, "-----BEGIN CERTIFICATE-----", "-----END CERTIFICATE-----");

    struct hey_setup_config setup_conf;

    //setup_conf.cert_key.dev_cert = ...
    // set the contents of the certificate according to your product application.
    // ...
    setup_conf.cert_key.dev_private_key = (uint8_t *)dev_private_key;
    setup_conf.cert_key.dev_private_key_len = dev_private_key_len;
    setup_conf.cert_key.dev_cert = (uint8_t *)dev_cert;
    setup_conf.cert_key.dev_cert_len = dev_cert_len;
    setup_conf.cert_key.ecdh_private_key = (uint8_t *)ecdh_private_key;
    setup_conf.cert_key.ecdh_private_key_len = ecdh_private_key_len;
    setup_conf.cert_key.product_cert = (uint8_t *)product_cert;
    setup_conf.cert_key.product_cert_len = product_cert_len;
    setup_conf.timeout_seconds = 15 * 60;
    setup_conf.cbs = &setup_cbs;
    hey_log_printf(HEY_LOG_LEVEL_INFO, "Start to setup !");
    hey_setup_start(&setup_conf);

    free(dev_private_key);
    free(dev_cert);
    free(ecdh_private_key);
    free(product_cert);
}

static void user_application_run(void)
{
    if (hey_setup_get_state() != HEY_SETUP_STATE_SUCCESS) {
        device_start_to_setup();
    }
    while (1) {
        vTaskDelay(1);
    }
}

void user_task(void *pvParameter)
{
    user_application_run();
    vTaskDelete(NULL);
}
